@@ -1,9 +1,13 @@
 import operator
 import sys
 PY3 = sys.version_info[0] > 2
+PY33 = sys.version_info[0] == 3 and sys.version_info[1] == 3
+PY34 = sys.version_info[0] == 3 and sys.version_info[1] == 4
+PYPY = hasattr(sys, 'pypy_version_info')
 
-__all__ = ('PY3', 'map', 'filter', 'range', 'zip', 'reduce', 'zip_longest',
-           'iteritems', 'iterkeys', 'itervalues', 'filterfalse')
+__all__ = ('map', 'filter', 'range', 'zip', 'reduce', 'zip_longest',
+           'iteritems', 'iterkeys', 'itervalues', 'filterfalse',
+           'PY3', 'PY34', 'PYPY')
 
 if PY3:
     map = map
